#!/usr/bin/perl

use warnings;
use strict;

my $BERYL_COMMAND = "/usr/bin/beryl-manager";

main();

sub main
{
   if (fork() > 0)
   {
      return;
   }

   $SIG{USR1} = \&restartBeryl;
   $SIG{HUP} = \&restart;

   while (1)
   {
      if (isBerylCrashed())
      {
         killBerylManager();
      }
      if (!isBerylManagerRunning())
      {
         startBeryl();
      }
      sleep(2);
   }
}

sub stopBeryl
{
   system("killall beryl beryl-manager");
}

sub startBeryl
{
   system($BERYL_COMMAND);
}

sub restartBeryl
{
   stopBeryl();
   sleep(1);
   startBeryl();
}

sub restart
{
   exec(@ARGV);
}

sub isBerylManagerRunning
{
   return (0 != getBerylCount());
}

sub getBerylCount
{
   my @berylCount = `ps auwx | grep -v grep | grep beryl-manager`;
   return scalar(@berylCount);
}

sub isBerylCrashed
{
   my $dir = "/tmp";
   opendir(DIR, "$dir");
   my @files = readdir(DIR);
   closedir(DIR);

   @files = grep(/beryl_crash/, @files);
   for my $file (@files)
   {
      $file = "$dir/$file";
      logMessage("Deleting crash file $file");
      unlink($file);
   }

   return scalar(@files);
}

sub killBerylManager
{
   system("killall beryl-manager");
}

sub logMessage
{
   my $message = shift;

   open(LOG, ">>/tmp/beryl.log");
   print LOG "$message\n";
   close(LOG);
}
